/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define VER     "0.1"
#ifndef u_char
    #define u_char  unsigned char
#endif



u_char *jbmail_decpwd(u_char *data, int size);
u_char *jbmail_encpwd(u_char *data, int size);



int main(int argc, char *argv[]) {
    u_char  *result;

    setbuf(stdout, NULL);

    fputs("\n"
        "JBMail saved password encoder/decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\nUsage: %s <dec/enc> <password>\n"
            "\n"
            "Decoding/Encoding:\n"
            "0 = decode the stored password (that in the file jbmail.ini)\n"
            "1 = encode a plain-text password\n"
            "\n", argv[0]);
        exit(1);
    }

    if(argv[1][0] & 1) {    // encoding
        result = jbmail_encpwd(argv[2], strlen(argv[2]));
        printf("\nEncoded password: %s\n\n", result);
        free(result);
    } else {                // decoding
        result = jbmail_decpwd(argv[2], strlen(argv[2]));
        printf("\nDecoded password: %s\n\n", result);
        free(result);
    }

    return(0);
}



u_char *jbmail_decpwd(u_char *data, int size) {
    static u_char   *buff;
    u_char          *ptr;
    int             i;

    size >>= 1;
    buff = malloc(size + 1);
    if(!buff) return("");

    ptr = buff;
    for(i = 0; i < size; i++) {
        *ptr++ = (((*data++ - 1) << 4) | (*data++ - 'A')) - i - size;
    }
    *ptr = 0;
    return(buff);
}



u_char *jbmail_encpwd(u_char *data, int size) {
    static u_char   *buff;
    u_char          *ptr,
                    tmp;
    int             i;

    buff = malloc((size << 1) + 1);
    if(!buff) return("");

    ptr = buff;
    for(i = 0; i < size; i++, data++) {
        tmp = *data + i + size;
        *ptr++ = (tmp >> 4) + 'A';
        *ptr++ = (tmp & 0xf) + 'A';
    }
    *ptr = 0;
    return(buff);
}



